#include "lcd.h"

#include <avr/io.h>
#include <util/delay.h>

void writeCommand(uint8_t);
void writeData(uint8_t);
void writeByte(uint8_t);
void toggleE(void);
void write8BitMode(uint8_t b);

//----------------------------------------------------------------------------//
//  basic initialisation
//----------------------------------------------------------------------------//
void lcd_init()
{
    // port setup
    LCD_PORT.OUT = 0x00;
    LCD_PORT.DIRSET = 0xff;  //output
    
    // power on delay
    _delay_ms(400);
               
    // activate 8 bit mode 
    write8BitMode(0x30); 
    _delay_ms(15);
    // repeat twice
    write8BitMode(0x30);
    _delay_us(200);
    write8BitMode(0x30);
    _delay_us(200);
        
    // switch to 4 bit mode
    write8BitMode(0x20);    //function set: 4bit, 2lines
    _delay_us(200);
       
    // basic configuration
    
    writeByte(0x2C);    //function set: 4bit, 2lines, extension=1 -> 4lines    
    _delay_us(50);
    
    //writeByte(0x09);
    //_delay_us(50);
    
    writeByte(0x0c);			// display on/off control
	_delay_us(50);
	writeByte(0x06);			// entry mode set
	_delay_ms(5);
    
    
    
	lcd_clear();
    lcd_home();
    
}

//----------------------------------------------------------------------------//
//  clear display
//----------------------------------------------------------------------------//
void lcd_clear()
{
    writeCommand(0x01);
}

//----------------------------------------------------------------------------//
//  goto 0,0
//----------------------------------------------------------------------------//
void lcd_home()
{
    writeCommand(0x02);
}

void lcd_write(char *s)
{
    char c;
    while ( (c = *s++) ) {
        writeData(c);
    }
}

void lcd_goto(uint8_t x, uint8_t y)
{
    #if LCD_LINES==2
    if ( y==0 )
        writeCommand((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else
        writeCommand((1<<LCD_DDRAM)+LCD_START_LINE2+x);
    #endif
    #if LCD_LINES==4
    if ( y==0 )
        writeCommand((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else if ( y==1)
        writeCommand((1<<LCD_DDRAM)+LCD_START_LINE2+x);
    else if ( y==2)
        writeCommand((1<<LCD_DDRAM)+LCD_START_LINE3+x);
    else /* y==3 */
        writeCommand((1<<LCD_DDRAM)+LCD_START_LINE4+x);
    #endif
}


void writeCommand(uint8_t cmd)
{
    LCD_PORT.OUTCLR = LCD_RS;		// set RS line low 
    _delay_us(50);
    writeByte(cmd);
    _delay_ms(5);   //5ms CMD 
}

void writeData(uint8_t data)
{
    LCD_PORT.OUTSET = LCD_RS;
    _delay_us(50);
    writeByte(data);
    _delay_us(200);   //200us data
    
}

//----------------------------------------------------------------------------//
//  write a databyte to the lcd
//----------------------------------------------------------------------------//


void write8BitMode(uint8_t b)
{
    uint8_t data;
    
    //dont touch unused pins
    data = LCD_PORT.OUT & 0xf0;
    
    //upper nibble
    LCD_PORT.OUT = data | ((b>>4) & 0x0f);
    toggleE();
    _delay_ms(5);
    
}
void writeByte(uint8_t b)
{
    uint8_t data;
    
    //dont touch unused pins
    data = LCD_PORT.OUT & 0xf0;
    
    //upper nibble
    LCD_PORT.OUT = data | ((b>>4) & 0x0f);
    toggleE();
    
    //lower nibble
    LCD_PORT.OUT = data | (b & 0x0f);
    toggleE();
	//_delay_us(200);
}

void toggleE()
{
    LCD_PORT.OUTSET = LCD_E;		
	_delay_us(5);
	LCD_PORT.OUTCLR = LCD_E;		
    _delay_us(5);
}
